<?php
/*******************************************
 @File		allsites.php
 @Comment	PHP wrapper around Planetlab's sites.xml
 			returns a (lat,long) pair given a &hostname=<hostname>
 			returns a .kml file suitable for GE placing all sites, given &kmz

 @written	6/23/06, 14:15
 @Author	derDoc
 @email		derDoc@gmx.de
 @Note		Needs PHP > version 5, because of XML stuff
 
 Changelog:
-> v0.1: 23.6.2k6, 14:15
	-initial version
	-gets and saves sites.xml
	-parses DOM into array
	-outputs KML for GE
-> v0.2: 25.6.2k6, 11:00
	-zipping of kml works, but not on every box (Linux)
	-GE does not seem to like zipped file, even when proper Content-type is set (ideas?)
	-moved everything else into functions
-> v0.3: 25.6.2k6, 18:00
	-added functionality for requesting (long,lat) tuple for hostname
	-uses XPath for avoiding parsing everything into an array
-> v0.4: 27.6.2k6, 02:15
	-added <Snippet> to the KML
	-changed names from parts in kml
	-name is now SHORT_NAME
	-snippet is NAME
	-and the description has the FULL_SITE_NAME (is it different from NAME, anywhere?)
	-added SITE_IDs to names in side_bar and overlay
	-added UTEP icon to Document
	
******************************************/


define('SITESXML_LOCALNAME','sites.xml');
define('SITESXML_HOST','planet-lab.org');
define('SITESXML_PATH','/xml/sites.xml');
//set if you want debug output (1=msgs,2=all) if not overriden
if (!defined('DEBUG')) define('DEBUG',0);
//set error_reporting according to debug
if (DEBUG==2) error_reporting(E_STRICT | E_ALL);
else if (DEBUG==1) error_reporting(E_COMPILE_ERROR | E_ERROR | E_CORE_ERROR);
else error_reporting(0);
if (substr(PHP_VERSION,0,1) < 5)
	die("PHP < ver5 not supported!") ;

//-------------------------------------------------------------------

if (file_exists(SITESXML_LOCALNAME) && is_file(SITESXML_LOCALNAME))
	$modtime = date('r',filemtime(SITESXML_LOCALNAME));

$allsites = stripped_http_get(SITESXML_HOST, SITESXML_PATH,80,$modtime);

if ($allsites){
	if (!file_exists(SITESXML_LOCALNAME))
		touch(SITESXML_LOCALNAME) or
			die("Couldn't create local copy of ".SITESXML_LOCALNAME."! Does PHP have the right to create files in PWD? If not please touch ".SITESXML_LOCALNAME);
	file_put_contents(SITESXML_LOCALNAME,$allsites);
}else if(!file_exists(SITESXML_LOCALNAME))
		exit(1);
	
	

//start getting the XML
$doc = new DOMDocument();
$doc->load(SITESXML_LOCALNAME);


if (isset($_GET['kmz']) || isset($_POST['kmz'])){
	error_reporting(0);
//set appropiate content-type, GE does not really care if set
//	header('Content-type: application/vnd.google-earth.kml+xml kml');
//Note: GE does not seem to like a zipped KML file even if this header is sent:
//	header('Content-type: application/vnd.google-earth.kmz kmz');
	$sites = parseXMLtoArray($doc);
	$KML = buildKML($sites);

//	echo zip($KML);
	echo $KML;

}else if (isset($_GET['hostname']) || isset($_POST['hostname'])){
	if (!($h = $_GET['hostname'])) $h = $_POST['hostname'];
	$xpath = new DOMXPath($doc);
	$xpath_lat = "//HOST[@NAME='".$h."']/../@LATITUDE";
	$xpath_lon = "//HOST[@NAME='".$h."']/../@LONGITUDE";

	$e_lat = $xpath->query($xpath_lat);
	$e_lon = $xpath->query($xpath_lon);
   	$ret = 	"(" . $e_lon->item(0)->nodeValue . "," . $e_lat->item(0)->nodeValue . ")";
	echo $ret;	
}else
	echo "Operation not supported";



/***************** filter string through zip and return zipped, return $in untouched if fails ******************/
/************ !!! BEWARE !!! Function does not work on some linuxes, it hangs forever !!! BEWARE !!! ******************/

function zip($in){

$descriptorspec = array(
		0 => array("pipe", "r"),  // stdin is a pipe that the child will read from
		1 => array("pipe", "w"),  // stdout is a pipe that the child will write to
		2 => array("pipe", "w") 
	);
	$process = proc_open('zip', $descriptorspec, $pipes, NULL, NULL);
	
	if (is_resource($process)) {
		stream_set_blocking($pipes[0], FALSE);
		stream_set_blocking($pipes[1], FALSE);
		stream_set_blocking($pipes[2], FALSE);

		while ($wr = fwrite($pipes[0], $in));
		fflush($pipes[0]);
		fclose($pipes[0]);
		
		while ($out[] = stream_get_contents($pipes[1]));
		fclose($pipes[1]);
//		echo "error:" . stream_get_contents($pipes[2]) . "\n";
		fclose($pipes[2]);
		$return_val = proc_close($process);
//		echo "ret: " . $return_val . "\n";
	}else return $in;
return join('',$out);
}

/***************** parse the XML Doc into an array ******************/

function parseXMLtoArray($doc){
$sites = array();

//and iterate over all sites
foreach ($doc->getElementsByTagName('SITE') as $site_item) {
   	$site_full_name = $site_item->getAttributeNode('FULL_SITE_NAME')->nodeValue;
   	$site = array();
   	foreach ($site_item->attributes as $att)
   		$site[$att->nodeName] = $att->nodeValue;
	$hosts = array();
	
	foreach ($site_item->getElementsByTagName('HOST') as $host_item) {
			$host = array();
			$host_name = $host_item->getAttributeNode('NAME')->nodeValue;
   		foreach ($host_item->attributes as $att)
   			$host[$att->nodeName] = $att->nodeValue;
		$site[$host_name] = $host;
		$hosts[$host_name] = $host;
	}
	$site['HOSTS'] = $hosts;
	
	$sites[$site_full_name] = $site;
}

//print_r($sites);
return $sites;

}


/***************** build the KML for GE ******************/

function buildKML($sites){

$allsite_xml_str = '';
$font_fam = "font-family:serif;font-style:italic;";
$pic_url = "http://dlab.robust.cs.utep.edu/~vitus/plmr/img/pl-logo-46x26.gif";


foreach ($sites as $site){

	$allsite_xml_str .= "
		<Placemark>
			<name>
				<![CDATA[
					(".$site['SITE_ID'].") ".$site['SHORT_SITE_NAME']."
				]]>
			</name>
			<Snippet>
				<![CDATA[
					".$site['NAME']."
				]]>
			</Snippet>
			<styleUrl>#PL-Style</styleUrl>";

	$allsite_xml_str .= "
			<description>
				<![CDATA[
					<h1><a href='".$site['URL']."'>".$site['FULL_SITE_NAME']."</a></h1>
					<p style='margin-left:1px;'>
						<table border=0>
							<tr>
								<td><b>Site ID</b></td>
								<td>".$site['SITE_ID']."</td>
							</tr>
							<tr>
								<td><b>Short Name</b></td>
								<td>".$site['SHORT_SITE_NAME']."</td>
							</tr>
							<tr>
								<td><b>Login Base</b></td>
								<td>".$site['LOGIN_BASE']."</td>
							</tr>
							<tr>
								<td><b>URL</b></td>
								<td><a href='".$site['URL']."'>".$site['URL']."</a></td>
							</tr>";
					foreach ($site['HOSTS'] as $host){
						$allsite_xml_str .= "
							<tr>
								<td><b>".$host['NAME']."</b></td>
								<td>
									<table border=0>
										<tr>
											<td style='".$font_fam."'>Node-ID</td>
											<td>".$host['NODE_ID']."</td>
										</tr>
										<tr>
											<td style='".$font_fam."'>Status</td>
											<td>".$host['STATUS']."</td>
										</tr>
										<tr>
											<td style='".$font_fam."'>Version</td>
											<td>".$host['VERSION']."</td>
										</tr>
										<tr>
											<td style='".$font_fam."'>BootState</td>
											<td>".$host['BOOT_STATE']."</td>
										</tr>
										<tr>
											<td style='".$font_fam."'>BootVersion</td>
											<td>".$host['BOOT_VERSION']."</td>
										</tr>
										<tr>
											<td style='".$font_fam."'>BW-Limit</td>
											<td>".$host['BWLIMIT']."</td>
										</tr>
										<tr>
											<td style='".$font_fam."'>IP</td>
											<td>".$host['IP']."</td>
										</tr>
										<tr>
											<td style='".$font_fam."'>Model</td>
											<td>".$host['MODEL']."</td>
										</tr>
									</table>	
								</td>
							</tr>";
						
					}

	$allsite_xml_str .= "
						</table>
					</p>
				]]>
			</description>
			<visibility>1</visibility>
			<open>0</open>
			<Point>
				<extrude>0</extrude>
				<tessellate>0</tessellate>
				<altitudeMode>clampToGround</altitudeMode>
				<coordinates>".$site['LONGITUDE'].",".$site['LATITUDE'].",0</coordinates>
	  		</Point>
		</Placemark>";
}

$resp = "<?xml version='1.0' encoding='UTF-8'?>
	<kml xmlns='http://earth.google.com/kml/2.0'>
	<Folder>
		<Style id='PL-Style'>
		 	<IconStyle id='PL-Icon-46x26'>
		  		<scale>0.7</scale>
				<Icon>
					<href>$pic_url</href>
				</Icon>
			</IconStyle>
		</Style>
  		<description>PlanetLab AllSites</description>
  		<name>AllSites</name>
  		<visibility>0</visibility>
  		<open>1</open>
  		".$allsite_xml_str."
	</Folder>
</kml>";

return $resp;
}

/***************** Utility Functions ******************/

function stripped_http_get($host='localhost', $path='/', $port=80, $ifmod_since=NULL){
	return strstr(raw_http_get($host,$path,$port,$ifmod_since),'<');
}

function raw_http_get($host='localhost', $path='/', $port=80, $ifmod_since){
		if ($port == 443) $tohost = "ssl://".$host;
		else $tohost = $host;
		$fp = fsockopen($tohost, $port, $errno, $errstr, 30);
		if ($fp){

			$out = "GET $path HTTP/1.1\r\n";
				$out .= "Host: $host\r\n";
				$out .= "Accept: */*\r\n";
				$out .= "Connection: close\r\n";
			if ($ifmod_since) $out .= "If-Modified-Since: $ifmod_since\r\n";
			$out .= "\r\n";
   			fwrite($fp, $out);
			
			$in = stream_get_contents($fp);
   			fclose($fp);
   			return $in;

		}else{
   			if (DEBUG) echo "conf->raw_http_get: <b>Couldn't open connection to $host:80$path <br>$errstr ($errno)<br /></b>".LINE_DELIM;
   		}
   		return false;
}



?>
